#! /usr/bin/env python
# coding: utf-8

# Stack_Layers_By_Colors.py release 2
# Created by Tin Tran http://bakon.ca/gimplearn/
# Comments directed to http://gimpchat.com or http://gimpscripts.com
#
# License: GPLv3
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY# without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# To view a copy of the GNU General Public License
# visit: http://www.gnu.org/licenses/gpl.html
#
# ----------------------------------------------------------------------------------
# Creates a new image of numbers of colors stacked from brightest on top to darkest on bottom based on number of colors specified.
# 
#
# ------------
#| Change Log |
# ------------
# Rel 1: Initial release.
# Rel 2: Added Reversable flag from Dark to light.
# Rel 3: 04/03/2023 add layers name by color, remowe selection, layers resize to image size, change of location in Gimp menu.

import math
import string
#import Image
from gimpfu import *
from array import array

def python_stack_layers_by_colors(image, layer, numofcolors, reverse) :
	#pdb.gimp_image_undo_group_start(image)
	pdb.gimp_context_push()
	
	#PUT YOUR CODE HERE
	
	gimp.progress_init("Stacking layers by colors...")
	
	#creates new image and attach it to a display
	new_image = pdb.gimp_image_new(layer.width,layer.height,RGB)
	new_display = pdb.gimp_display_new(new_image)
	
	#make copy of active layer on new image
	layer_copy = pdb.gimp_layer_new_from_drawable(layer,new_image)
	pdb.gimp_image_insert_layer(new_image,layer_copy,None,-1)
	
	
	pdb.gimp_image_convert_indexed(new_image,NO_DITHER,MAKE_PALETTE,numofcolors,0,1,"Custom Palette Name Ignored")
	(num_bytes,colormap) = pdb.gimp_image_get_colormap(new_image)
	
	
	pdb.gimp_image_convert_rgb(new_image)
	pdb.gimp_layer_add_alpha(layer_copy)
	
	pdb.gimp_context_set_antialias(1)
	pdb.gimp_context_set_feather(0)
	pdb.gimp_context_set_feather_radius(0,0)
	pdb.gimp_context_set_sample_threshold(0)
	for i in range(0,num_bytes/3):
		if reverse == 1:
			current_color = colormap[(num_bytes/3 - i - 1)*3:((num_bytes/3 - i - 1)*3)+3]
		else:
			current_color = colormap[i*3:(i*3)+3]
		pdb.gimp_image_select_color(new_image,CHANNEL_OP_REPLACE,layer_copy,current_color)
		selection,x1,y1,x2,y2 = pdb.gimp_selection_bounds(new_image)
		if selection == 1: #if there is a selection we create a new layer out of it.
			pdb.gimp_edit_copy(layer_copy)
			floating_sel = pdb.gimp_edit_paste(layer_copy,TRUE)
			sl=pdb.gimp_floating_sel_to_layer(floating_sel)
			pdb.gimp_layer_resize_to_image_size(floating_sel)					# add MrQ	
			pdb.gimp_item_set_name(floating_sel, "Color "+ str(current_color))  # add MrQ			
	gimp.progress_update(float(1)/10)
	pdb.gimp_selection_none(new_image)                                          # add MrQ
	pdb.gimp_context_pop()
	#pdb.gimp_image_undo_group_end(image)
	pdb.gimp_displays_flush()
    #return

register(
	"python_fu_stack_layers_by_colors",                           
	"Creates a New Image of Stacked Layers by Colors. Ver.3",
	"Creates a New Image of Stacked Layers by Colors...",
	"Tin Tran",
	"Tin Tran",
	"2016",
	"<Image>/Colors/Components/Stack Layers By Colors...",             #Menu path
	"RGB*, GRAY*", 
	[
	(PF_SPINNER, "numofcolors", "Number of Colors:", 16, (1, 256,1)),
	(PF_TOGGLE, "reverse",   "Reverse Order (From Dark to Light):", 0),
	],
	[],
	python_stack_layers_by_colors)

main()
